#!/usr/bin/env python

# Copyright 2020 Unlimited Robotics

import argparse
import os
import sys
import platform

from rayasdk.connect import URConnect
from rayasdk.container_handlers.vcs import URVCS, check_versions
from rayasdk.logger import log_error, log_verbose, set_logger_level
from rayasdk.scanner import URScanner
from rayasdk.initializer import URInitializer
from rayasdk.runner import URRunner
from rayasdk.simulator import URSimulator
from rayasdk.killer import URKiller
from rayasdk.constants import *
from rayasdk.utils import *


class URSDK:

    def __init__(self):
        self.command_objects = []
        self.command_objects.append(URInitializer())
        self.command_objects.append(URScanner())
        self.command_objects.append(URConnect())
        self.command_objects.append(URRunner())
        self.command_objects.append(URSimulator())
        self.command_objects.append(URKiller())
        self.command_objects.append(URVCS())
        self.init_parser()

    def init_parser(self):
        # Init top parser
        self.argparser = argparse.ArgumentParser(
            description='Unlimited Robotics SDK',
        )
        group = self.argparser.add_mutually_exclusive_group()
        group.add_argument("-v",
                           "--verbose",
                           action="store_true",
                           help="increase output verbosity.")
        group.add_argument("-q",
                           "--quiet",
                           action="store_true",
                           help="don't print on stdout.")
        subparsers = self.argparser.add_subparsers(help='SDK Command',
                                                   dest='command')
        subparsers.required = True
        # Init subparsers
        for obj in self.command_objects:
            obj.init_parser(subparsers)
        # Parse arguments
        self.args, self.unknownargs = self.argparser.parse_known_args()
        # Setup Logger
        set_logger_level(self.args.verbose, self.args.quiet)

    def check_global_folder(self):
        try:
            update_constants()
            # create folders that are needed in the .ur path
            os.makedirs(SIMS_APPS, exist_ok=True)
            os.makedirs(SIMS_DATA_APPS, exist_ok=True)
            os.makedirs(SIMS_DATA_APPS_DEVEL, exist_ok=True)
            # if ur_root not exist or is empty clone it
            if not os.path.exists(RAYAENV_DOCKER_UR_ROOT) or len(
                    os.listdir(RAYAENV_DOCKER_UR_ROOT)) == 0:
                log_info('Cloning raya_root_folder_templates...')
                return_code = subprocess.call((
                    f'{RAYAENV_DOCKER_UR_ROOT_REPO} {RAYAENV_DOCKER_UR_ROOT} && '
                    f'cd {RAYAENV_DOCKER_UR_ROOT} && cp env.template env'),
                                              shell=True,
                                              stderr=subprocess.PIPE)
                if return_code != 0:
                    log_error(
                        ('Error cloning the raya_root_folders_templates repo '
                         'check if you have permitions'))

        except OSError as exc:
            log_error(exc)
            return False
        return True

    def init_ursdk_folder(self):
        if not os.path.exists(URSDK_TEMP_PATH):
            log_verbose(
                f'Folder {URSDK_TEMP_PATH} does not exists, creating it.')
            try:
                os.mkdir(URSDK_TEMP_PATH)
            except OSError as exc:
                log_error(f'Folder {URSDK_TEMP_PATH} could not be created.')
                return False
            return True
        else:
            log_verbose(f'Folder {URSDK_TEMP_PATH} found.')
            return True

    def run(self):
        try:
            # Check for temp folder
            if not self.init_ursdk_folder():
                return False
            # Check .ur folder
            if not self.check_global_folder():
                log_error(f'Could not create .ur folder')
                return False
            # Do not check version on update
            if self.args.command != URVCS().COMMAND:
                # Check versions
                if not check_versions():
                    return False
            # Execute Command
            for obj in self.command_objects:
                if self.args.command == type(obj).COMMAND:
                    return obj.run(self.args, self.unknownargs)
        except Exception as e:
            log_error(str(e))
            return False


def main():
    if platform.system() not in ['Linux', 'Windows', 'Darwin']:
        print(f'Platform \'{platform.system()}\' not supported.')
        exit(1)
    ursdk = URSDK()
    ret = ursdk.run()
    if not ret:
        log_verbose('Finished with error.')
        sys.exit(1)
    else:
        sys.exit(0)


if __name__ == "__main__":
    main()
